#ifndef _MA_SDL_H_
#define _MA_SDL_H_

#include <stdlib.h>
#include "ma_object.h"
#include "utils/utils.h"

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_gfxPrimitives.h>
#include <SDL/SDL_mixer.h>

#define max(x,y) (x>=y)?x:y
#define min(x,y) (x<=y)?x:y

#define MAKERGBCOLOR(r,g,b) ( (r<<24)|(g<<16)|(b<<8)|0xFF )
#define MAKERGBACOLOR(r,g,b,a) ( (r<<24)|(g<<16)|(b<<8)|a )

#define GET_REDVALUE(color) ( (color>>24) & 0xFF )
#define GET_GREENVALUE(color) ( (color>>16) & 0xFF )
#define GET_BLUEVALUE(color) ( (color>>8) & 0xFF )
#define GET_ALPHAVALUE(color) ( color & 0xFF )

#define GP2X_BUTTON_UP              (0)
#define GP2X_BUTTON_DOWN            (4)
#define GP2X_BUTTON_LEFT            (2)
#define GP2X_BUTTON_RIGHT           (6)
#define GP2X_BUTTON_UPLEFT          (1)
#define GP2X_BUTTON_UPRIGHT         (7)
#define GP2X_BUTTON_DOWNLEFT        (3)
#define GP2X_BUTTON_DOWNRIGHT       (5)
#define GP2X_BUTTON_CLICK           (18)
#define GP2X_BUTTON_A               (12)
#define GP2X_BUTTON_B               (13)
#define GP2X_BUTTON_X               (15)
#define GP2X_BUTTON_Y               (14)
#define GP2X_BUTTON_L               (11)
#define GP2X_BUTTON_R               (10)
#define GP2X_BUTTON_HOME            (8)
#define GP2X_BUTTON_SELECT          (9)
#define GP2X_BUTTON_VOLUP           (16)
#define GP2X_BUTTON_VOLDOWN         (17)

// Definicin de Prototipos y Tipos de Datos, 
// para facilitar referencias cruzadas
//
class TMABitmap;
class TMAGame;
class TMAScene;
class TMAImageBank;
class TMASprite;
class TMAFont;
class TMASound;
class TMAOgg;
/*
class TMATTFont;
class TMAMidi;
class TMAOggMem;
*/

typedef TMABitmap *PMABitmap;
typedef TMAGame* PMAGame;
typedef TMAScene *PMAScene;
typedef TMAImageBank* PMAImageBank;
typedef TMASprite *PMASprite;
typedef TMAFont* PMAFont;
typedef TMASound* PMASound;
typedef TMAOgg*  PMAOgg;
/*
typedef TMATTFont* PMATTFont;
typedef TMAMidi* PMAMidi;
typedef TMAOggMem*  PMAOggMem;
*/


// **************************************
// Clase que encapsula un BITMAP: TMAGame
// **************************************
//
class TMABitmap : public TMAObject
{
    REGISTER_CLASS protected:
        
        friend class TMAGame;

        static PMAGame GMAGame;
        SDL_Surface *io_bmp;
        bool ib_destroy;
    
        TMABitmap(SDL_Surface*,bool shared=false);       
        
    public:
        PUBLIC_CLASS_ID;
        ~TMABitmap();

    // ***********************************************
    // Mtodos de creacin de BITMAPS/Buffers Graficos
    // ***********************************************
        
        // Crea un bitmap a partir de un archivo de imagen
        //
        static PMABitmap loadFromFile(const char *filename,bool compatible=false);
                
        // Crea un bitmap de la misma profundidad que la pantalla actual.
        // Puede crearse en memoria de video.
        //
        static PMABitmap create(int width,int height,bool videomem);
        
        // Crea un bitmap del tamao y profundidad especificados.
        //
        static PMABitmap create(int width,int height,int depth);
        
        // Crea un nuevo bitmap, del mismo tamao y caracteristicas
        // del indicado como argumento.
        //
        static PMABitmap create(PMABitmap);

        // Crea un nuevo bitmap, compartiendo memoria con otro existente.
        // El existente, nunca debe ser liberado antes que este.
        //
        static PMABitmap create(PMABitmap,int x,int y,int width,int height);

        // Devuelve la referencia al BITMAP interno..
        //
        inline SDL_Surface *getInternalBitmapRef(void) { return io_bmp; }

        // Devuelve una copia del bitmap
        //
        PMABitmap clone(void);
        
        // Devuelve una copia del bitmap, pero con flips
        //
        PMABitmap cloneFlipX(void);
        PMABitmap cloneFlipY(void);
        PMABitmap cloneFlipXY(void);        
                        
    // ****************************************
    // Mtodos para la obtencin de propiedades
    // ****************************************
        
        bool saveToFile(const char *filename);
        int getColorDepth(void);
        int getWidth(void);
        int getHeight(void);
        void getSize(int& width,int& height);

    // **************************************
    // Mtodos operaciones sobre el TMABitmap
    // **************************************

        void lock(void);
        void unlock(void);

        bool makeDisplayCompatible(void);
        bool makeDisplayCompatibleWithAlpha(void);

        bool copyPalette(const PMABitmap);
        void setTransparentColor(Uint32 color);
        void unsetTransparentColor(void);
        bool isTransparent(void);
        
        void clear(Uint8 red,Uint8 green,Uint8 blue);
        void clear(Uint32 color);
        
        void draw(const PMABitmap,int px,int py);
        void draw(const PMABitmap,int ox,int oy,int width,int height,int dx,int dy);
        void drawSprite(const PMABitmap,int px,int py,bool flipx=false,bool flipy=false);
        void drawSpriteColorized(const PMABitmap,int px,int py,bool flipx=false,bool flipy=false,int r=255,int g=255,int b=255);
        void drawSpriteShadowed(const PMABitmap spr,int px,int py,bool flipx=false,bool flipy=false,int alpha=150);
        void drawLine(int ox,int oy,int dx,int dy,Uint32 color);
        void drawRect(int ox,int oy,int dx,int dy,Uint32 color);
        void drawRectFilled(int ox,int oy,int dx,int dy,Uint32 color);
        void drawCircle(int cx,int cy,int radio,Uint32 color);
        void drawCircleFilled(int cx,int cy,int radio,Uint32 color);               
        void drawEllipse(int cx,int cy,int radiox,int radioy,Uint32 color);
        void drawEllipseFilled(int cx,int cy,int radiox,int radioy,Uint32 color);

        void   setPixel(int px,int py,Uint32 color);
        Uint32 getPixel(int px,int py);        

        // ------------ Operaciones Alpha
        //
        bool isAlpha(void);
        void setAlpha(int alpha);
        void disableAlpha(void);
        
        // ------------ Operaciones de Resize y Filtrado
        //
        void bilinearStretch(const PMABitmap,int ox,int oy,int owidth,int oheight,
                            int dx,int dy,int dwidth,int dheight);
};

// ***********************************
// Clase principal del nucleo: TMAGame
// ***********************************
//
class TMAGame : public TMAObject
{
    REGISTER_CLASS protected:

        friend class TMABitmap;
        
        static TMAGame* oc_MAGame;
        static bool ci_endgame;
        static bool ib_soundAvailable;
        static bool ib_joystickAvailable;
        static SDL_Surface *ScreenSurf;
        static PMABitmap Screen;
        static int ii_last_ascii_key;

        int ii_game_speed;
                
        PMAScene io_current_scene;
        PMAScene io_next_scene;
        bool ib_remove_last_scene;
        
        void setGP2XKey(int key,bool down);
        
    public:
        PUBLIC_CLASS_ID;
        static const int KEY_MAX = 512;
        static bool KeyState[KEY_MAX];
        static long il_time_millis;
        static int ARGC;
        static char **ARGV;

        // Constructor y Destructor
        //
        TMAGame(void);
        virtual ~TMAGame();
        
        // Limpia el buffer de teclas y los flags de estados
        //
        static void releaseKeys(void);
        
        // Devuelve un puntero a la instancia del juego actual.
        //
        static TMAGame *getCurrentGame(void);          
                
        // Detiene la ejecucin durante ciertos milisegundos
        //
        static void sleep(int ms);
        
        // Devuelve el tiempo actual en milisegundos
        //
        static long getCPUTime(void);
        
        // En este mtodo especificarse la rutina de inicializacin global 
        // del juego
        //
        virtual bool init(void) { return true; }
        
        // En este mtodo especificarse la rutina de limpieza global 
        // del juego
        //
        virtual void cleanup(void);
                  
        // Este mtodo se debe invocar para que comience el ciclo principal
        // del juego
        //
        void run(void);
        
        // Mtodo para mostrar mensajes de Error
        //
        static void showAlert(const char *);      // Muestra una alerta.
        static void showLastAlert();        // Muestra la ultima que genero un error.
        void showFatalError(const char *);        // Muestra una alerta y cierra la aplicacin.
         
        // Establece o Recupera el escenario del juego
        //
        bool setScene(PMAScene,const PMAObject params = NULL,bool remove_last=true);
        const PMAScene getScene(void) { return io_current_scene; }
         
    // ***********************************
    // Mtodos de control sobre el entorno
    // *********************************** 
        
        // Incializa el modo grafico.
        //
        bool setGraphicsMode(int width,int height,int colordepth,bool fullscreen);
        bool setFullScreen(bool fs);
        bool isFullScreen(void);
        
        // Establece el titulo de la ventana principal.
        //   
        void setWindowTitle(const char *);
        
        // Devuelve el tamao ancho/alto de pantalla establecido
        //
        const PMABitmap getScreen(void) { return Screen; }
        void getScreenSize(int& width,int& height);
        int getScreenWidth(void);
        int getScreenHeight(void);
        int getScreenDepth(void);
        void FlipScreen(void);
        
    // ***********************************
    // Mtodos de control sobre el flujo
    // *********************************** 
    
        // Establece la velocidad de refresco del juego en Frames por Segundo
        // por defecto se asume 0 fps, lo que hace que exista la mxima 
        // velocidad posible.
        //
        void setGameFPS(int fps);

        // Este mtodo se invoca en cada iteracin del juego.
        // La idea es introducir aqui llamadas a calculos o controles.
        // Si existe una escena asignada, TMAScene, se invoca el doRun 
        // de la escena, en otro caso, se invoca el del juego.
        //
        virtual void doRun(void){};

        // Este mtodo se invoca en cada iteracin del juego para
        // que se dibuje lo que sea necesario.
        // Si existe una escena asignada, TMAScene, se invoca el doPaint
        // de la escena, en otro caso, se invoca el del juego.
        //
        virtual void doPaint(TMABitmap&) {};
        
        // Este mtodo termina con la ejecucin del juego.
        //
        static void endGame(void);

    // *******************************************
    // Mtodos de control sobre sonido y joystick
    // *******************************************
    
        static bool isSoundAvailable(void) { return ib_soundAvailable; }
        static void setSoundAvailable(bool av) {  ib_soundAvailable=av;}

        static bool isJoystickAvailable(void) { return ib_joystickAvailable; }
        static void setJoystickAvailable(bool av) {  ib_joystickAvailable=av;}

    // *****************************
    // Mtodos del control de idioma
    // *****************************
    
        static void setupLanguage(const char *domain,const char *locale);
        
    // Obtiene el valor ascii de la ultima tecla pulsada, si aun lo esta
    // en otro caso, de vuelve 0.
    
        static int getLastAsciiKeyPressed(void);
        static void releaseLastAsciiKeyPressed(void);
        
    // Establece el volumen
    
        void setVolume(int v);    
    
};

// ****************************************
// Clase que encapsula una escena: TMAScene
// ****************************************
//
class TMAScene : public TMAObject
{
    REGISTER_CLASS protected:
                
        bool ib_initialized;
        int ii_Estado;
        
    public:
        PUBLIC_CLASS_ID;
        TMAScene();
        
        virtual bool doInit(const PMAObject params=NULL);
        
        // Este mtodo se invoca en cada iteracin del juego.
        // La idea es introducir aqui llamadas a calculos o controles.
        //
        virtual bool doRun(void){ return false; };
        
        // Este mtodo se invoca en cada iteracin del juego para
        // que se dibuje lo que sea necesario.
        //
        virtual void doPaint(TMABitmap&){};
        
        bool isInitialized(void) { return ib_initialized; }
};        

// *************************************************
// Clase que encapsula un BANCO de IMAGENES: TMAGame
// *************************************************
//
class TMAImageBank : public TMAObject
{
    REGISTER_CLASS protected:
        
        TMAImageBank();
        TMAVector vector;
        PMABitmap imgsource;
        
    public:
        PUBLIC_CLASS_ID;
        
        ~TMAImageBank();
        
        // Crea un banco de imgenes de tamao fijo a partir de un TMABitmap
        // La imagen original no se destruye, pero una vez creado el banco
        // tampoco se hace referencia a ella.
        //
        static PMAImageBank createStatic(TMABitmap&,int tx,int ty,int ax,int ay,bool separator=false);
        
        // Crea un banco de imgenes de tamao fijo a partir de un archivo de imagen
        // 
        static PMAImageBank createStatic(const char *filename,int tx,int ty,int ax,int ay,bool separator=false);
        
        // Crea un banco de imgenes donde cada imagen puede ser de un tamao, a partir
        // de un bitmap y un vector de objetos (TMARectangle)
        //
        static TMAImageBank *createDynamic(TMABitmap&,TMAVector& zonas);
        
        // Crea un banco de imgenes donde cada imagen puede ser de un tamao, a partir
        // de un bitmap y un vector de objetos (TMARectangle)
        //
        static TMAImageBank *createDynamic(const char *filename,TMAVector& zonas);
        
        // Devuelve una imagen del banco de imgenes, que esta indexada
        // por el indice especificado.
        //
        const PMABitmap getBitmap(int index);
        
        // Devuelve el numero de entradas en el banco.
        //
        long size(void);
        
        // Devuelve una copia de este image bank
        //
        //PMAImageBank clone(void);
};

// ****************************************
// Clase que encapsula un sprite: TMASprite
// ****************************************
//
class TMASprite : public TMAObject
{
    REGISTER_CLASS protected:
 
        bool ib_valido;                 // Indica si es valido
        int  ii_Frame;                  // Fotograma actual
        int  ii_Retardo;                // Retardo de fotograma
        int  ii_RetardoPos;             // Contador de Retardo
        const int *ia_AnimSequence;     // Secuencia de animacin actual
        int  ii_AnimSequenceLength;     // Secuencia de animacin actual
        int  ii_EstadoAnim;             // Estado de la animacin actual
        int  ii_Estado;                 // Estado general del Sprite
        int  ii_CurrentFotograma;       // Fotograma Actual
        bool ib_flip_x;                 // Sprite invertido X
        bool ib_flip_y;                 // Sprite invertido Y
        
        PMAImageBank io_images;   // Banco de Imgenes
        
    public:
        PUBLIC_CLASS_ID;
        TMARectangle position;        
        
        TMASprite(PMAImageBank);
        virtual ~TMASprite(){}
       
        bool isValid(void)      { return ib_valido;}
        void validate(void)     { ib_valido=true; }
        void invalidate() 		{ ib_valido=false; }
        void setFlipX(bool b)	{ ib_flip_x = b; }
        void setFlipY(bool b)	{ ib_flip_y = b; }
        bool isFlippedX(void)   { return ib_flip_x; }
        bool isFlippedY(void)   { return ib_flip_y; }
        
        virtual void animate(void);
        virtual bool isCollisioningWith(TMASprite& s);
        virtual void putAt(int x,int y) { position.x = x; position.y = y; }	
        virtual bool draw(TMABitmap&);
        
        void setAnimSequence(const int *,int length);
};

// *****************************************************
// Clase que encapsula un tipo de letra grafico: TMAFont
// *****************************************************
//
class TMAFont : public TMAObject
{
    REGISTER_CLASS protected:
        
        PMAImageBank io_bank;
        int width,height;
        int ii_spacex,ii_transcolor;
        const char *is_chars;
        
    public:
        PUBLIC_CLASS_ID;
        
        TMAFont();
        ~TMAFont();
        
        // Carga un tipo de letra.
        //
        bool load(const char *fontFile,int width,int height,int col,int row,
                  bool pixel_separator=false,int transcolor=0);
	        
        // Establece el espacio horizontal entre caracteres.
        //
        void setSpaceBetweenChars(int);
        
        // Establece el juego de caracteres
        //
        void setCharacters(const char *);
        
        // Devuelve la anchura de un texto
        //
        int getTextWidth(const char *);
        
        // Devuelve la altura de un texto
        //
        int getTextHeight(void);
        
        // Dibuja un texto sobre un bitmap.
        // Si X=-1, se centra el texto en horizontal, asumiendo que la anchura
        // de la pantalla se indica en el parametro "scrwidth"
        // El parametro offsety, puede apuntar a un array de enteros que se modificaran
        // la posicin Y de cada letra. Debe contener tantos elementos como la anchura
        // de la pantalla.
        //
        void drawText(TMABitmap&,const char *text,int x, int y,int *offsety=NULL,
                      int scrwidth=0);
                      
        void setAlpha(int);
        void disableAlpha(void);
};

// ***************************************
// Clase que encapsula un sonido: TMASound
// ***************************************
//
class TMASound : public TMAObject
{
    REGISTER_CLASS protected:
        
        Mix_Chunk *psound; 
        int ii_voice;   
        
    public:
        PUBLIC_CLASS_ID;
        
        TMASound();
       ~TMASound();
       
        bool load(const char *filename);
        void unload(void);
        void play(bool loop=false,int volume=32,int pan=127);
        void stop(void);
        bool isPlaying(void);
};

// **************************************************
// Clase que encapsula un sonido Ogg en disco: TMAOgg
// **************************************************
//
class TMAOgg : public TMAObject
{
    REGISTER_CLASS protected:
                
        Mix_Music *psound; 
                
    public:
        PUBLIC_CLASS_ID;
        
        TMAOgg();
       ~TMAOgg();
       
        bool load(const char *filename);
        void close(void);
        void play(bool loop=false,int volume=32);
        void stop(void);
        bool isPlaying(void);
};

/*
// **********************************************
// Clase que encapsula un tipo de letra true type
// **********************************************
//
class TMATTFont : public TMAObject
{
    REGISTER_CLASS protected:

        ALFONT_FONT *font;
        
    public:
        PUBLIC_CLASS_ID;

        TMATTFont();
        ~TMATTFont();

        // Carga un tipo de letra.
        //
        bool load(const char *fontFile,int size=10);

        // Establece el tamao del tipo de letra
        //
        void setSize(int size);

        // Devuelve la anchura de un texto
        //
        int getTextWidth(const char *);

        // Devuelve la altura de un texto
        //
        int getTextHeight(void);

        void drawText(TMABitmap&,const char *text,int x, int y,int color,bool antialias=false);
        void drawTextCenter(TMABitmap&,const char *text,int x, int y,int color,bool antialias=false);
        void drawTextRight(TMABitmap&,const char *text,int x, int y,int color,bool antialias=false);
};


// *********************************************
// Clase que encapsula una melodia midi: TMAMidi
// *********************************************
//
class TMAMidi : public TMAObject
{
    REGISTER_CLASS protected:
        
        static MIDI* sp_current_midi;
        MIDI *pmidi;    
        void clear(void);
        
    public:
        PUBLIC_CLASS_ID;
        
        TMAMidi();
       ~TMAMidi();
       
        bool load(const char *filename);
        void play(bool loop=false);
        void stop(void);
        void pause(void);
        void resume(void);
};


// ****************************************************
// Clase que encapsula un sonido Ogg en memoria: TMAOgg
// ****************************************************
//
class TMAOggMem : public TMAObject
{
    REGISTER_CLASS protected:
        
        ALOGG_OGG *ogg;
        void *buffer;
        long position;
        bool ploop;
        int pvolume;
                
    public:
        PUBLIC_CLASS_ID;
        
        TMAOggMem();
       ~TMAOggMem();
       
        bool load(const char *filename);
        void close(void);
        void play(bool loop=false,int volume=512);
        void setVolume(int volume);
        bool doPlay(void);
        void stop(void);
        void pause(void);
        void resume(void);
        bool isPlaying(void);
};



*/

#endif
